"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const fs = require("fs");
const constants_1 = require("./constants");
const logger_1 = require("./lib/logger");
class GitSentinel {
    constructor(cfg) {
        this.changedFiles = [];
        this.errors = [];
        this.statuses = [];
        const config = cfg || this.findConfig();
        if (!config) {
            logger_1.default.error('sentinel.json not found');
            process.exit(1);
        }
        this.settings = config;
    }
    /**
     * Ищем sentinel.json по дереву каталогов вверх,
     * когда нашли, парсим его и возвращаем
     */
    findConfig() {
        let dir = __dirname;
        // console.log({ dir });
        while (dir.length) {
            dir = dir.substr(0, dir.lastIndexOf(constants_1.default.dirDelimiter));
            if (fs.existsSync(`${dir}/sentinel.json`)) {
                const config = JSON.parse(fs.readFileSync(`${dir}/sentinel.json`).toString());
                return config;
            }
        }
    }
    /**
     * Запуск всего
     */
    fire() {
        this.changedFiles = this.getChangedFiles();
        this.writeChangedFiles();
        for (const rule of this.settings.rules) {
            const firedFiles = this.conditionMet(this.changedFiles, rule.mask);
            if (firedFiles.length) {
                this.fireRule(rule, firedFiles);
            }
        }
        this.writeErrors();
        return this.errors.length ? 1 : 0;
    }
    /**
     * Красивый вывод файлов, измененных в процессе коммита
     */
    writeChangedFiles() {
        if (this.changedFiles.length) {
            logger_1.default.log('\x1b[36m', 'Файлы');
            for (const file of this.changedFiles) {
                process.stdout.write(`- ${file} \n`);
            }
            process.stdout.write('\n\n');
        }
        else {
            logger_1.default.log('Файлы не были изменены');
        }
    }
    /**
     * Красивый вывод ошибок
     */
    writeErrors() {
        if (this.errors.length) {
            logger_1.default.log('\x1b[31m', 'Ошибки');
            for (const err of this.errors) {
                process.stdout.write(`${err.cmd} : \n`);
                process.stdout.write(err.message);
                process.stdout.write('\n\n');
            }
            logger_1.default.log('\x1b[0m');
        }
        else {
            logger_1.default.log('\x1b[32m', 'Ошибок нет');
            logger_1.default.log('\x1b[0m');
        }
    }
    /**
     * Выбирает из списка всех измененных в коммите файлов только те, которые соответсвуют маске
     *
     * @param changedFiles string[] - все измененные в коммите файлы
     * @param mask string - регулярка от правила
     *
     * @returns string[]
     */
    conditionMet(changedFiles, mask) {
        return changedFiles.filter((el) => {
            const match = el.match(new RegExp(mask));
            if (match && match.length > 0) {
                return true;
            }
            return false;
        });
    }
    /**
     * Выполняет одно правило по списку соответстующих этому правилу файлов
     * В процесе дергает (возможно много раз) this.runCmd
     *
     * @param rule IRule - правило для старта
     * @param files string[] - список файлов, измененных в этом коммите
     */
    fireRule(rule, files) {
        let filesToFire = files;
        if (!rule.separate) {
            filesToFire = [''];
        }
        const commands = rule.commands;
        for (const file of filesToFire) {
            for (const cmd of commands) {
                const success = this.runCmd(this.getCmdForFilename(file, cmd));
                if (!success && rule.stopOnErrors) {
                    return;
                }
            }
        }
    }
    /**
     * Запускаем команду, и добавляем ее вывод в this.statuses
     *
     * @param cmd string - текст команды
     *
     * @returns boolean - true, если команда выполнена успешно
     */
    runCmd(cmd) {
        let message = '';
        let success = true;
        try {
            const cmdRet = child_process_1.execSync(cmd, { stdio: ['pipe', 'pipe', 'ignore'] });
            message = cmdRet.toString().trim();
        }
        catch (err) {
            success = false;
            message = err.output[1].toString().trim();
        }
        this.addStatus(cmd, success, message);
        return success;
    }
    /**
     * Сохраняем статус выполнения команды
     *
     * @param cmd string - выполненная команда
     * @param success boolean - статус завершения
     * @param message string - сообщение
     */
    addStatus(cmd, success, message) {
        this.statuses.push({ cmd, success, message });
        if (!success) {
            this.errors.push({ cmd, message });
        }
    }
    /**
     * Возвращает список измененных в этом коммите файлов
     */
    getChangedFiles() {
        const ret = child_process_1.execSync('git diff HEAD --name-only');
        const newFiles = ret.toString().split('\n').filter(e => e);
        return newFiles;
    }
    /**
     * Подставляем название файла в команду, заменяя ${filename} на название файла
     *
     * @param filename - название файла
     * @param cmd - команда
     */
    getCmdForFilename(filename, cmd) {
        return cmd.replace('${filename}', filename);
    }
}
exports.default = GitSentinel;
//# sourceMappingURL=gitSentinel.js.map